package com.simonstl.fragment;

import org.xml.sax.*;
import org.xml.sax.helpers.DefaultHandler;
import java.io.IOException;
import java.util.*;

/*
The contents of this file are subject to the Mozilla Public License  Version 1.1 (the "License"); you may not use this file except in  compliance with the License. You may obtain a copy of the License at 
http://www.mozilla.org/MPL/ 
Software distributed under the License is distributed on an "AS IS"  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the  License for the specific language governing rights and limitations under  the License. 

The Original Code is available at  http://simonstl.com/projects/fragment/original. 

The Initial Developer of the Original Code is Simon St.Laurent. Portions  created by Simon St.Laurent are Copyright (C) 2001. All Rights Reserved. 

Contributor(s): 
*/

/**
*
* This class creates different types of DocComponent objects from a SAX event, or directly through a list of properties.
*
* @version 0.01 15 July 2001
* @author Simon St.Laurent
*/

public class ComponentFactory {
    
    final static int MATCH=0;
    final static int SPLIT=1;
    final static int SERIES=2;

public static DocComponent createComponent (String uri, String local,
        String qName, Attributes atts)  {

DocComponent result=null;

        if (uri.equals("http://simonstl.com/ns/fragments/")) {

           if (local.equals("element")) {
              result=createElement(uri, local, qName, atts);
           }


           if (local.equals("attribute")) {
              result=createAttribute(uri, local, qName, atts);
           }

           if (local.equals("skip")) {
              result=createSkip(uri, local, qName, atts);
           }

           if (local.equals("chars")) {
              result=createCharacters(uri, local, qName, atts);
           }

	  }
return result;
}



public static Element createElement (String uri, String local, String qName, Attributes atts)  {
    Element newElement=null;

        if (uri.equals("http://simonstl.com/ns/fragments/")) {
           if (local.equals("element")) {
              String nsURI=atts.getValue("nsURI");
              String localName=atts.getValue("localName");
              String prefix=atts.getValue("prefix");

		  String before=atts.getValue("before");
              String after=atts.getValue("after");
		  String beforeInside=atts.getValue("beforeInside");
              String afterInside=atts.getValue("afterInside");
              newElement=createElement(nsURI, localName, prefix, before, after, beforeInside, afterInside);
           }


	  }
  return newElement;
}

public static Element createElement(String _nsURI, String _localName, String _prefix, String _before, String _after, String _beforeInside, String _afterInside) {
      Element newElement=new Element(_nsURI, _localName, _prefix);
	newElement.setBefore(_before);
      newElement.setAfter(_after);
	newElement.setBeforeInside(_beforeInside);
      newElement.setAfterInside(_afterInside);

  return newElement;
}


public static Attribute createAttribute (String uri, String local, String qName, Attributes atts)  {
    Attribute newAttribute=null;

        if (uri.equals("http://simonstl.com/ns/fragments/")) {
           if (local.equals("attribute")) {
              String nsURI=atts.getValue("nsURI");
              String localName=atts.getValue("localName");
              String prefix=atts.getValue("prefix");
              String type=atts.getValue("type");
              if (type==null) {type="CDATA";}
              String content=atts.getValue("content");
              newAttribute=createAttribute(nsURI, localName, prefix, type, content);
           }


	  }
  return newAttribute;
}

public static Attribute createAttribute(String _nsURI, String _localName, String _prefix, String _type, String _content) {
      Attribute newAttribute=new Attribute(_nsURI, _localName, _prefix);
	newAttribute.setType(_type);
      newAttribute.setContent(_content);

  return newAttribute;
}


public static Skip createSkip (String uri, String local, String qName, Attributes atts)  {
    Skip newSkip=null;

        if (uri.equals("http://simonstl.com/ns/fragments/")) {
           if (local.equals("skip")) {
              String before=atts.getValue("before");
              String after=atts.getValue("after");
              newSkip=createSkip(before, after);
           }


	  }
  return newSkip;
}

public static Skip createSkip(String _before, String _after) {
      Skip newSkip=new Skip();
      newSkip.setBefore(_before);
      newSkip.setAfter(_after);

  return newSkip;
}

public static Characters createCharacters (String uri, String local, String qName, Attributes atts)  {
    Characters newCharacters=null;

        if (uri.equals("http://simonstl.com/ns/fragments/")) {
           if (local.equals("chars")) {
              String before=atts.getValue("before");
              String after=atts.getValue("after");
              newCharacters=createCharacters(before, after);
           }


	  }
  return newCharacters;
}

public static Characters createCharacters(String _before, String _after) {
      Characters newCharacters=new Characters();
      newCharacters.setBefore(_before);
      newCharacters.setAfter(_after);

  return newCharacters;
}



   
}