package com.simonstl.fragment;

import java.util.ArrayList;
import java.util.Iterator;

/*
The contents of this file are subject to the Mozilla Public License  Version 1.1 (the "License"); you may not use this file except in  compliance with the License. You may obtain a copy of the License at 
http://www.mozilla.org/MPL/ 
Software distributed under the License is distributed on an "AS IS"  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the  License for the specific language governing rights and limitations under  the License. 

The Original Code is available at  http://simonstl.com/projects/fragment/original. 

The Initial Developer of the Original Code is Simon St.Laurent. Portions  created by Simon St.Laurent are Copyright (C) 2001. All Rights Reserved. 

Contributor(s): 
*/

/**
* <p>The ComponentList class extends ArrayList with functions for matching objects</p>
*
* <p>Version 0.02 adds an empty <code>matchRules()</code> method and a <code>repeats</code> boolean.</p>
* <p>Version 0.01 was a placeholder built solely on ArrayList .</p>
*
* @version 0.02 13 July 2001
* @author Simon St.Laurent
*/

public class ComponentList extends ArrayList {
   protected boolean repeats=false;

/**
* The matchRules() method accepts a component - Element or Attribute - and asks its contents to indicate 
* if they match that component.  If they do, this method will return true.
*/
public boolean matchRules (DocComponent testComponent, int matchType) {
   boolean result=false;
   int numberRules=this.size();

   for (int i=0;i<numberRules;i++) {
     DocComponent rule=(DocComponent) this.get(i);
     if (rule.matchComponent(testComponent, matchType)) {
       result=true;
     }
   }
   return result;
}
/**
* The getComponent() method returns a DocComponent and supports the repeats functionality.
*/

public DocComponent getComponent(int which) {
  if (repeats) {
    int currentSize=this.size();
    which=which%currentSize;
    if (which<0) {which=which+currentSize;}
  }
  return (DocComponent) this.get(which);
}


public void setRepeats(boolean _repeats) {
   repeats=_repeats;
}

public boolean getRepeats(){
   return repeats;
}

}