package com.simonstl.fragment;

import org.apache.xerces.parsers.SAXParser;
import org.xml.sax.Attributes;
import org.xml.sax.helpers.DefaultHandler;
import org.xml.sax.SAXParseException;
import org.xml.sax.SAXException;
import java.io.IOException;
import java.util.*;
import com.megginson.sax.XMLWriter;
import com.megginson.sax.DataWriter;
import java.io.FileReader;
import org.xml.sax.XMLReader;
import org.xml.sax.InputSource;
import org.xml.sax.helpers.ParserAdapter;
import org.xml.sax.helpers.XMLReaderFactory;

/*
The contents of this file are subject to the Mozilla Public License  Version 1.1 (the "License"); you may not use this file except in  compliance with the License. You may obtain a copy of the License at 
http://www.mozilla.org/MPL/ 
Software distributed under the License is distributed on an "AS IS"  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the  License for the specific language governing rights and limitations under  the License. 

The Original Code is available at  http://simonstl.com/projects/fragment/original. 

The Initial Developer of the Original Code is Simon St.Laurent. Portions  created by Simon St.Laurent are Copyright (C) 2001. All Rights Reserved. 

Contributor(s): 
*/

/**
* <p>This class just takes two arguments - a rules file and a target file.
* It loads the rules, feeds them to the filter, and sets up an XMLWriter
* so that testers can see the results on the command line.</p>
*
* <p>This class requires David Megginson's XMLWriter class, available from:
* <a href="http://www.megginson.com/Software/index.html">http://www.megginson.com/Software/index.html</a>.</p>
*
* <p>Version 0.06 added command-line options for the System properties used to specify the match and split processors.</p>
*
* <p>Version 0.05 added defaults for the System properties used to specify the match and split processors.</p>
*
* <p>Version 0.04 added command line options for printing out the rules and for specifying parsers.</p>
*
* <p>Version 0.03 moved all (non-regex) Xerces dependencies into this class, preparing for their removal.</p>
*
* <p>Version 0.02 switched to the DataWriter implementation of XMLWriter, which makes it easier to
* see results by adding whitespace.</p>
*
* @version 0.06 13 July 2001
* @author Simon St.Laurent
*/

public class FilterTester {
    static final int ELEMENTS_ONLY=0;
    static final int SINGLETONS_ONLY=1;
    static final int ALL=2;
    private static int matchLevel=ELEMENTS_ONLY;
/**
printHelp() lists options if the user doesn't specify any files.
*/

public static void printHelp() {
    System.out.println("FilterTester requires two arguments - a rules file and then a target XML file");
    System.out.println("");
    System.out.println("Options may appear before those arguments:");
    System.out.println("-m [parser class name]  - use the match processor specified");
    System.out.println("-p [parser class name] - specifies a SAX2 parser for use with this tool");
    System.out.println("-r  - parse and display the rules only");
    System.out.println("-s [parser class name]  - use the split processor specified");
    System.out.println("-ss [parser class name]  - use the series processor specified (not implemented)");
    System.out.println("More information is available at http://simonstl.com/projects/fragment/");
    System.out.println("");
/*
    System.out.println("");
    System.out.println("");
    System.out.println("");
    System.out.println("");
*/
}

/**
FilterTester's main method requires two arguments - a rules file and then a target XML file.

Options may appear before those arguments:<br />
-m [parser class name]  - use the match processor specified<br />
-p [parser class name] - specifies a SAX2 parser for use with this tool<br />
-r  - parse and display the rules only<br />
-s [parser class name]  - use the split processor specified<br />
-ss [parser class name]  - use the series processor specified (not implemented)<br />

*/

  // Main Method
  public static void main (String[] args) throws Exception {

    int numArgs=args.length;
    boolean printRules=false;
    String rulesFile;
    String targetFile;

//deal with System properties
   if ((System.getProperty("org.xml.sax.driver"))==null) {
      System.setProperty("org.xml.sax.driver","org.apache.xerces.parsers.SAXParser");
   }

   if ((System.getProperty("com.simonstl.fragment.matchProc"))==null) {
      System.setProperty("com.simonstl.fragment.matchProc","com.simonstl.fragment.XercesMatch");
   }

   if ((System.getProperty("com.simonstl.fragment.splitProc"))==null) {
      System.setProperty("com.simonstl.fragment.splitProc","com.simonstl.fragment.RegexpSplit");
   }

   if ((System.getProperty("com.simonstl.fragment.seriesProc"))==null) {
      System.setProperty("com.simonstl.fragment.seriesProc","com.simonstl.fragment.XercesMatch");
   }

   String parserName=System.getProperty("org.xml.sax.driver");

    if (numArgs<2) {
      printHelp();
    } else {
//process arguments
       rulesFile=args[numArgs-2];
       targetFile=args[numArgs-1];
for (int i=0; i<numArgs; i++) {
   if (args[i].equals ("-p")) {
       parserName=args[i+1];
       i++;
   }
   if (args[i].equals ("-p")) {
       String matchProc=args[i+1];
       System.setProperty("com.simonstl.fragment.matchProc",matchProc);

       i++;
   }
   if (args[i].equals ("-s")) {
       String splitProc=args[i+1];
       System.setProperty("com.simonstl.fragment.splitProc",splitProc);
       i++;
   }
   if (args[i].equals ("-ss")) {
       String seriesProc=args[i+1];
       System.setProperty("com.simonstl.fragment.seriesProc",seriesProc);
       i++;
   }
   if (args[i].equals ("-r")) {
       printRules=true;
       rulesFile=args[i+1];
       i++;
   }
}

       //  Create a SAX Parser
       XMLReader rulesParser = XMLReaderFactory.createXMLReader(parserName); 


        RulesLoader thisRulesLoader = new RulesLoader (rulesFile, rulesParser);
        FragmentRules rules=thisRulesLoader.rules;
        rules.setMatchLevel(matchLevel);

        XMLReader parser = XMLReaderFactory.createXMLReader(parserName); 
        FragmentFilter fragFilter=new FragmentFilter(parser);
        fragFilter.setRules(rules);
if (printRules) {
rules.print();
} else {
      DataWriter w = new DataWriter(fragFilter);
	w.parse(new InputSource(new FileReader(targetFile)));
      }
    }
  }
}