package com.simonstl.fragment;

import java.util.*;
import org.xml.sax.Attributes;


/*
The contents of this file are subject to the Mozilla Public License  Version 1.1 (the "License"); you may not use this file except in  compliance with the License. You may obtain a copy of the License at 
http://www.mozilla.org/MPL/ 
Software distributed under the License is distributed on an "AS IS"  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the  License for the specific language governing rights and limitations under  the License. 

The Original Code is available at  http://simonstl.com/projects/fragment/original. 

The Initial Developer of the Original Code is Simon St.Laurent. Portions  created by Simon St.Laurent are Copyright (C) 2001. All Rights Reserved. 

Contributor(s): 
*/

/**
* <p>This class stores a set of rules in a HashMap.  The namespaceURI and localName
* are combined to provide a key value.  This may eventually be extended to support
* attribute possibilities.</p>
*
* <p>Version 0.06 cleans up methods that required an {@link com.simonstl.fragment.Element Element} when 
* the current approach prefers {@link com.simonstl.fragment.DocComponent DocComponent}.  This will lead 
* to rules for processing attribute values as well as element content.</p>
*
* <p>Version 0.05 adds support for print() and toString().</p>
*
* <p>Version 0.04 now returns {@link com.simonstl.fragment.DocComponent DocComponent} objects more effectively, uses the <code>cleanDuplicate()</code> method to ensure that object reuse doesn't mar results.  It also checks to see that their results (content results, anyway, not yet attributes) are processed, permitting recursive processing.</p>
*
* <p>Version 0.03 is moving toward a fully encapsulated version where it is possible to determine
* if a rule exists and where DocComponents come in and leave.</p>
*
* <p>Version 0.02 moved some of the logic for key generation to the Element class.
* Making that logic work more smoothly will require changes to isRule() and getRule(), 
* as well as the {@link com.simonstl.fragment.FragmentFilter FragmentFilter} class.</p>
*
* @version 0.06 5 July 2001
* @author Simon St.Laurent
*/

public class FragmentRules {

private boolean matchAttributes=true;
final int ELEMENTS_ONLY=0;
final int SINGLETONS_ONLY=1;
final int ALL=2;
private int matchLevel=ALL;
HashMap rules;

public FragmentRules(){
   rules=new HashMap();
}

public void addRule(DocComponent newComponent, FragmentRule rule) {

  String key=newComponent.getKey();
  rules.put(key, rule);
//int size=rules.size();
//System.out.println("size: " +size);
}

public boolean isRule(String nsURI, String localName, Attributes atts) {
   boolean match=false;
if (matchLevel==ELEMENTS_ONLY) {
   match=isRule(nsURI, localName);
  }
  return match;
}

public boolean isRule(String nsURI, String localName) {
  String key=nsURI+">"+localName;
  return rules.containsKey(key);
}

public boolean isRule(DocComponent component) {
  return false;
}

public void removeRule(DocComponent newComponent) {
  String key=newComponent.getKey();
  rules.remove(key);
}

public FragmentRule getRule(String nsURI, String localName) {
  String key=nsURI+">"+localName;
  FragmentRule test= (FragmentRule) rules.get(key);
  return test;
}

public void setMatchAttributes(boolean _matchAttributes) {
   matchAttributes=_matchAttributes;
}

public boolean getMatchAttributes() {
   return matchAttributes;
}

public void processComponent(DocComponent targetComponent) {
   ArrayList localResults=new ArrayList();

   String nsURI=targetComponent.getNsURI();
   String localName=targetComponent.getLocalName();
if (isRule(nsURI,localName)) { 

   String matchString=targetComponent.getContent();

   FragmentRule currentRule=this.getRule(nsURI,localName);

   localResults=currentRule.processResult(matchString);

   targetComponent.clearContent();

   int numResults=currentRule.getSize();
   for (int i=0; i<numResults; i++) {
      DocComponent currentComponent=(DocComponent) localResults.get(i);
      targetComponent.addContent(currentComponent);
   }//end for
//next need to process child components
     targetComponent.processComponentContent(this);
}//end if
// System.out.println("Free!");

}


public void setMatchLevel(int level) {
   matchLevel=level;
}

public void print(){
/*System.out.println("Fragment Rule set");
   Iterator it=rules.keySet().iterator();
   while (it.hasNext()) {
    String key=(String)it.next();
    System.out.println("applies to: "+key);
    FragmentRule rule=(FragmentRule)rules.get(key);
    rule.print();
    System.out.println("-----------------------------------------------");
   }//end while
*/
System.out.println(toString());
}

public String toString(){
String value="Fragment Rule set:\n";
   Iterator it=rules.keySet().iterator();
   while (it.hasNext()) {
    String key=(String)it.next();
    value=value+("applies to: "+key+"\n");
    FragmentRule rule=(FragmentRule)rules.get(key);
    value=value+rule.toString()+"\n";
    value=value+("-----------------------------------------------\n");
   }//end while
return value;
}

}