 package com.simonstl.fragment;

import java.util.*;
import org.xml.sax.*;

/*
The contents of this file are subject to the Mozilla Public License  Version 1.1 (the "License"); you may not use this file except in  compliance with the License. You may obtain a copy of the License at 
http://www.mozilla.org/MPL/ 
Software distributed under the License is distributed on an "AS IS"  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the  License for the specific language governing rights and limitations under  the License. 

The Original Code is available at  http://simonstl.com/projects/fragment/original. 

The Initial Developer of the Original Code is Simon St.Laurent. Portions created by Simon St.Laurent are Copyright (C) 2001. All Rights Reserved. 

Contributor(s): 
*/

/**
* RegexProcFactory creates processors which allow developers to plug in alternative packages
* for processing regular expressions. 
*
* <p>The System properties identify which classes are used:</p>
* <ul>
* <li><code>com.simonstl.fragment.matchProc</code> - for match-style fragmentation</li>
* <li><code>com.simonstl.fragment.splitProc</code> - for split-style fragmentation</li>
* <li><code>com.simonstl.fragment.seriesProc</code> - for lex-style fragmentation (<em>not implemented!</em></li>
* </ul>
*
*<p>Version 0.03 makes this a Factory class, creating instances of classes instead of wrapping them here.</p>
*
*<p>Version 0.02 starts the move toward plug-in functionality by adding the createSplitter() and createMatcher() methods, 
* but they aren't used yet.</p>
*
* @version 0.03 11 July 2001
* @author Simon St.Laurent
*/


public class RegexProcFactory {
    static final int MATCH=0;
    static final int SPLIT=1;
    static final int SERIES=2;




/**
* The createRegEx() method returns an object which implements the {@link com.simonstl.fragment.RegExInt RegExInt} interface for match() processing using a regex package wrapped in the object identified by the <code>com.simonstl.fragment.matchProc</code>, <code>com.simonstl.fragment.splitProc</code>, or <code>com.simonstl.fragment.seriesProc</code> property.
*/

public static RegExInt createRegEx(int split, String pattern) {
  String propertyName="";
  RegExInt processor=null;
  if (split==SPLIT) {
     propertyName="com.simonstl.fragment.splitProc";
  } else if (split==MATCH){
     propertyName="com.simonstl.fragment.matchProc";

  } else if (split==SERIES){
     propertyName="com.simonstl.fragment.seriesProc";

  }//end else of isSplit

  String className = System.getProperty(propertyName);
  try {
     processor= createRegEx(className);
  } catch (Exception e) {
System.out.println(e);
System.exit(0);

  }
  processor.setPattern(pattern);
  return processor;
}

/**
* Attempt to create a from a class name.
*
* <p>Given a class name, this method attempts to load
* and instantiate the class as a Matcher.</p>
*
* @return An object supporting the {@link com.simonstl.fragment.RegExInt RegExInt} interface.
* @exception org.xml.sax.SAXException If the class cannot be
*            loaded, instantiated, and cast to RegExInt.
*/
protected static RegExInt createRegEx (String className) throws Exception {
	try {
	    return(RegExInt)(Class.forName(className).newInstance());
	} catch (ClassNotFoundException e1) {
	    throw new SAXException("RegExInt class " + className +
				   " not found", e1);
	} catch (IllegalAccessException e2) {
	    throw new SAXException("RegExInt class " + className +
				   " found but cannot be loaded",e2);
	} catch (InstantiationException e3) {
	    throw new SAXException("RegExInt class " + className +
				   " loaded but cannot be instantiated (no empty public constructor?)", e3);
	} catch (ClassCastException e4) {
	    throw new SAXException("RegExInt " + className +
				   " does not implement Matcher", e4);
	}
    }



}