package com.simonstl.fragment;

import org.xml.sax.*;
import org.xml.sax.helpers.DefaultHandler;
import java.io.IOException;
import java.util.*;

/*
The contents of this file are subject to the Mozilla Public License  Version 1.1 (the "License"); you may not use this file except in  compliance with the License. You may obtain a copy of the License at 
http://www.mozilla.org/MPL/ 
Software distributed under the License is distributed on an "AS IS"  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the  License for the specific language governing rights and limitations under  the License. 

The Original Code is available at  http://simonstl.com/projects/fragment/original. 

The Initial Developer of the Original Code is Simon St.Laurent. Portions  created by Simon St.Laurent are Copyright (C) 2001. All Rights Reserved. 

Contributor(s): 
*/

/**
* <p>This class loads the information stored in a rules document for use in fragment
* filtering.  It's based on Ethan Cerami's Xerces Quickstart, available at:</p>
* <p><a href="http://ecerami.com/xerces/" target="_top">http://ecerami.com/xerces/</a></p>
*
* 
*
*<p>Version 0.12 gives the factories even more control over object creation.</p>
*
*<p>Version 0.11 delegates most object creation to factories.</p>
*
*<p>Version 0.10 changed split="true|false" to type="match|split|series".</p>
*
*<p>Version 0.09 adds support for attributes in target elements.</p>
*
*<p>Version 0.08 adds support for attributes in result elements.</p>
*
* <p>Version 0.07 supports the move to {@link com.simonstl.fragment.DocComponent DocComponent} from {@link com.simonstl.fragment.Element Element}.  
* Note the use of the {@link com.simonstl.fragment.Characters Characters} and {@link com.simonstl.fragment.Skip Skip}
* classes.  Also note that this move is currently partial - no support yet for non-Element targets.</p>
*
* <p>Version 0.06 added support for the <code>beforeInside</code> and <code>afterInside</code> attributes
* of the <code>resultElement</code> element.</p>
*
* <p>Version 0.05 added support for the <code>chars</code> element 
* and the <code>before</code> and <code>after</code> attributes,
* and modified how support for the <code>skip</code> element
* worked.  Elements with empty <code>nsURI</code> and <code>localName</code> attributes
* are now treated as <code>chars</code> elements, not 
* <code>skip</code> elements.</p>
*
* <p>Version 0.04 added support for the <code>pattern</code> attribute, replacing <code>matchPattern</code>, 
* and added support for the <code>split</code> attribute
* of the <code>fragmentRule</code> element.</p>
*
* <p>Version 0.03 added support for the <code>skipFirst</code> and <code>repeat</code> attributes
* of the <code>fragmentRule</code> element.</p>
*
* <p>Version 0.02 added support for the <code>skip</code> element, which permits rules to skip portions
* of a match. (By default the first match is skipped.) </p> 
*
* @version 0.11 13 July 2001
* @author Simon St.Laurent
*/

public class RulesLoader extends DefaultHandler {
    
    public FragmentRules rules;   //set of rules
    private FragmentRule newRule; //current rule
    protected static ArrayList appliesTo;
    private Element parentElement=null;
    private boolean inAppliesTo=true;
    final int MATCH=0;
    final int SPLIT=1;
    final int SERIES=2;

    // Constructor
    public RulesLoader (String xmlFile, XMLReader parser) {
        
        //  Set Content Handler
        parser.setContentHandler (this);

        // Create a rules container
        rules=new FragmentRules();

 
        //  Parse the Document      
        try {
            parser.parse(xmlFile);
        } catch (SAXException e) {
            System.err.println (e);
        } catch (IOException e) {
            System.err.println (e);
        }
    }
    
    //  Start Element Event Handler
public void startElement (String uri, String local,
        String qName, Attributes atts)  {

       
        if (uri.equals("http://simonstl.com/ns/fragments/")) {
//System.out.println(qName);
           if (local.equals("fragmentRule")) {
             newRule=FragmentRuleFactory.createRule(uri, local, qName, atts);
             appliesTo=new ArrayList();

           } else if (local.equals("appliesTo")) {
             inAppliesTo=true;
           } else if (local.equals("produce")) {
             inAppliesTo=false;
           } else {
             DocComponent newComponent=ComponentFactory.createComponent(uri, local, qName, atts);
             if (newComponent!=null) {
//element
               if (newComponent instanceof Element) {
                 parentElement=(Element) newComponent;
               }
               if (newComponent instanceof Attribute) {
                if (parentElement==null) {
                 if (inAppliesTo) {
                   appliesTo.add(newComponent);
                 } else {
                   newRule.addResult(newComponent);
                 }
              } else {
                parentElement.addAtt((Attribute)newComponent);
              }

               } else {
                if (inAppliesTo) {
                  appliesTo.add(newComponent);
                } else {
                  newRule.addResult(newComponent);
                }//end inAppliesTo or not
              }//end not an attribute

       }//end component not-null if
     }//end generic else
  }//end namespace
}//end startElement

    public void endElement (String uri, String local,
        String qName)  {

        if (uri.equals("http://simonstl.com/ns/fragments/")) {
           if (local.equals("element")) {
           parentElement=null;
           }

           if (local.equals("fragmentRule")) {
             inAppliesTo=true;  
             
              //time to clean up all the fragment rule list
              int size=appliesTo.size();
              for (int i=0; i<size; i++){
               DocComponent currentComponent=(DocComponent)appliesTo.get(i);
//              String currentComponentName=currentComponent.toString(); 
//System.out.println("applies to: " + currentComponentName);
               newRule.addAppliesTo(currentComponent);
//will need to obliterate currentComponent from method below
               rules.addRule(currentComponent, newRule);
              }
           }
	  }
    }

   
}